      *> Copyright (c) 2005 - 2020 Veryant. Users of isCOBOL
      *> may freely modify and redistribute this program.

       IDENTIFICATION DIVISION.

       CLASS-ID. MYCALLHANDLER as "MYCALLHANDLER" 
                               implements jCallHandler.

       ENVIRONMENT DIVISION.

       CONFIGURATION SECTION.

       REPOSITORY.
           class JLangObect     as "java.lang.Object[]"
           class Jboolean       as "boolean"
           class jIscobolCall   as "com.iscobol.rts.IscobolCall"
           class jCallOverflowException 
                             as "com.iscobol.rts.CallOverflowException"
           class jCallHandler   as "com.iscobol.rts.CallHandler"
           class jexception     as "java.lang.Exception"
           .

       IDENTIFICATION DIVISION.
       FACTORY.
       WORKING-STORAGE SECTION.
       PROCEDURE DIVISION.

       END FACTORY.

       IDENTIFICATION DIVISION.
       OBJECT.
       WORKING-STORAGE SECTION.
       01 ws-calling-program pic x any length.
       01 ws-start-time      pic 9(8).
       01 ws-end-time        pic 9(8).

       PROCEDURE DIVISION.

      *    This method is executed before the execution of each called  
      *    program
       IDENTIFICATION DIVISION.
       METHOD-ID. IS-BEFORECALL AS "beforeCall".

       WORKING-STORAGE SECTION.
       77  obj object reference.

       LINKAGE SECTION.
       77  iscobol-call OBJECT REFERENCE jIscobolCall.
       77  j-lang-object OBJECT REFERENCE JLangObect.

       PROCEDURE DIVISION USING iscobol-call j-lang-object 
                        RAISING jCallOverflowException.

       MAIN.
           set obj to iscobol-call
           if obj:>getClass:>getName:>startsWith("com.iscobol.lib")
           else
              accept ws-start-time from time

              display " " upon sysout
              display "HOOK" upon sysout
              display "before call of " obj:>getClass:>getName 
                                            upon sysout
              display "Start time : " ws-start-time upon sysout


           end-if
           .
       END METHOD.

      *    This method is executed at the end of the excution of each 
      *    program
       IDENTIFICATION DIVISION.
       METHOD-ID. IS-AFTERCALL as "afterCall".

       WORKING-STORAGE SECTION.
       77  obj object reference.

       LINKAGE SECTION.
       77  iscobol-call   object reference jIscobolCall.
       77  j-lang-object  object reference JLangObect.

       procedure division using iscobol-call j-lang-object 
                        raising jCallOverflowException.

       MAIN.
           set obj to iscobol-call
           if obj:>getClass:>getName:>startsWith("com.iscobol.lib")
           else
              accept ws-end-time from time
              initialize ws-calling-program 
              call "c$calledby" using ws-calling-program
              display " " upon sysout
              display "HOOK" upon sysout
              display "After call of " obj:>getClass:>getName 
                                            upon sysout
              display "End time : " ws-end-time upon sysout
              display "Called by : " ws-calling-program upon sysout
           end-if.
       end method.

      *    This method is executed before the cancel statement of a  
      *    specific program
       IDENTIFICATION DIVISION.

       METHOD-ID. IS-BEFORECANCEL AS "beforeCancel".

       WORKING-STORAGE SECTION.
       77  return-field object reference jboolean.
       77  obj object reference.

       LINKAGE SECTION.
       77 iscobol-call object reference jIscobolCall.

       PROCEDURE DIVISION USING iscobol-call 
                      RETURNING return-field.

       MAIN.
           set obj to iscobol-call
           set RETURN-FIELD to true.
       END METHOD.

      *    This method is executed after the cancel statement of a  
      *    specific program
       IDENTIFICATION DIVISION.
       METHOD-ID. IS-AFTERCANCEL AS "afterCancel".

       WORKING-STORAGE SECTION.
       77  obj object reference.

       LINKAGE SECTION.
       77  iscobol-call object reference jIscobolCall.

       PROCEDURE DIVISION USING iscobol-call.

       MAIN.
           set obj to iscobol-call
       end method.

      *    This method is executed before the cancel all statement 
       IDENTIFICATION DIVISION.

       METHOD-ID. IS-BEFORECANCELALL AS "beforeCancelAll".

       WORKING-STORAGE SECTION.
       77  return-field object reference jboolean.

       PROCEDURE DIVISION RETURNING return-field.

       MAIN.
           set return-field to true.
       end method.

      *    This method is executed after the cancel all statement 
       IDENTIFICATION DIVISION.
       METHOD-ID. IS-AFTERCANCELALL as "afterCancelAll".

       WORKING-STORAGE SECTION.
       PROCEDURE DIVISION.

       MAIN.
       end method.

       END OBJECT.
